/****************************************************************************
** Leistungsorientierte Krankenanstaltenfinanzierung (LKF)                 **
** Scoring-C-Sourcen V2024.0                                               **
** (C) 2023 BMSGPK                                                         **
****************************************************************************
** utils5.c - Basisfunktionen                                              **
**                                                                         **
**		 Alex.Holzer@chello.at                                             **
**       Bernhard.Pesec@dothealth.at                                       **
***************************************************************************/


#include "lkfint5.h"
#include <ctype.h>
#include <math.h>

static long str2int (const char *str, unsigned int len);


/* function IsEmpty **************************************************
**    Liefert 0, wenn text (Maximallnge len) nicht leer ist, sonst **
**    1.                                                            **
*********************************************************************/

int IsEmpty (const char *text, int len)
{
	while (len && *text == ' ')
		{ ++text; --len; }
	if (!len || !*text) return 1;
	else return 0;
}

/* function IsNull ***************************************************
**    berprft, ob in einem String der Wert 0 steht.               **
*********************************************************************/

static int IsNull (const char *text, int len)
{
	int i;
	for (i = 0; i < len && text[i] == ' '; i++);
	for (; i < len && text[i] == '0'; i++);
	for (; i < len && text[i] == ' '; i++);
	if (i == len) return 1;
	else return 0;
}

/* function UCase ****************************************************
**     Konvertiert einen String auf Grossbuchstaben                 **
*********************************************************************/

void UCase(char *ps, int len)
{
	for (; len > 0; len--) ps[len-1] = (char) toupper(ps[len-1]);
}


int CompareUCase (const char *cand, const char *patt, int len)
{
	int i;
	for (i = 0; i < len; i++) {
		if (toupper(cand[i]) != patt[i]) return 1;
	}
	return 0;
}

/* function Util_Datum_Nullen *****************************************
**   Diese Funktion fllt das Datum mit Nullen                       **
**   an den leeren Stellen auf, und setzt Null-stellenDatum auf leer **
**********************************************************************/

void Util_Datum_Nullen(char *pDest, const char *pSrc)
{
	memmove (pDest, pSrc, 8);
	if (pDest[4] == ' ' && pDest[5] >= '0' && pDest[5] <= '9') pDest[4] = '0';
	if (pDest[6] == ' ' && pDest[7] >= '0' && pDest[7] <= '9') pDest[6] = '0';

	if (IsNull (pDest, 8)) memset (pDest, ' ', 8);
}

/** function Util_FC_Nullen *******************************
**     Diese Funktion fllt den FC mit Nullen            **
**     an der 7. und 8. Stelle auf                       **
***********************************************************/

void Util_FC_Nullen(char *pDest, const char *pSrc)
{
	int i, j;
	char tmp_buf[9];

	for (i = 0; i < 8 && pSrc[i] == ' '; i++) ;
	for (j = i; j < 8 && pSrc[j] != ' '; j++) pDest [j-i] = pSrc[j];
	j -= i; 
	for (i = j; i < 8; i++) pDest[i] = ' ';

/*	if (j == 6) memcpy (pDest + 6, "00", 2); */

	memmove (tmp_buf, pDest, 8);
	tmp_buf[8] = 0;
	if (IsNull (tmp_buf, 8)) memset (pDest, ' ', 8);
}

/* function GetInt ***************************************************
**    berprft, ob in text in der Lnge len ein Integer-Wert steht	**
**    (return 1) oder nicht (return 0). Ist resultat kein NULL-Ptr, **
**    so wird der Integer-Wert darin zurckgeliefert.               **
*********************************************************************/

_SCORING_EXPORT int lkf_GetInt (const char *text, int len, int *resultat)
{
	int digit = 0, minus = 0, erg;
	while (len && *text == ' ') { --len; ++text; }

	if (len	&& (*text == '-' || *text == '+'))
	{ 
		if (*text == '-') minus = 1;
		--len; ++text; 
		while (len && *text == ' ') { --len; ++text; }
	}

	erg = 0;

	while (len && isdigit (*text)) 
	{ 
		erg = erg * 10 + (*text - '0');
		--len; 
		++text; 
		digit = 1; 
	}

	while (len && *text == ' ') { --len; ++text; }

	if ((!len || !*text) && digit) {
	
		if (resultat) {
			*resultat = minus ? -erg : erg;
			return 1;
		}
		else return minus ? -erg : erg;
	
	}
	else {
		if (resultat) *resultat = 0;
		return 0;
	}
}

double simVal(const char *str, int strlen)
{
/* der Versuch die Basic Val Funktion nachzuempfinden*/
/* Basic Val skipt alle Char zwischen 9-13*/
/* akzeptiert leading Vorzeichen + -*/
/* kennt . als kommaseperator*/
/* beendet scan bei jedem nicht nummerischen Zeichen*/
/* und liefert alles bis dahin gefundene*/
char Buffer[100]; int BufZeiger=0;
int Vorzeichen=0/*FALSE*/; int Number=0/*FALSE*/;
int i;

	if (strlen > 99) return 0.0; /* Zahl zu gro (atof cant handle more)*/

	for (i = 0; i < strlen; i++)
	{
		if (*(str+i) == ' ') continue; /* skip blank*/
		if ((*(str+i) >= 9) && (*(str+i) <= 13)) continue; /* skip it*/
		if ((*(str+i) >= '0') && (*(str+i) <= '9')) {Buffer[BufZeiger++] = *(str+i);Number=(-1)/*TRUE*/;continue;}
		if (*(str+i) == '.')                        {Buffer[BufZeiger++] = *(str+i);Number=(-1)/*TRUE*/;continue;}
		if ((*(str+i) == '-') || (*(str+i) == '+'))
		{
			if (Vorzeichen == (-1)/*TRUE*/) break; /* kein zweites vorzeichen*/
			if (Number == (-1)/*TRUE*/) break;     /* kein vorzeichen nach num.Werten*/
			Buffer[BufZeiger++] = *(str+i);
			Vorzeichen = (-1)/*TRUE*/;
			continue;
		}
		break;   /* falsches Zeichen */
	}
	Buffer[BufZeiger] = 0; /* append ascii Null*/

	return (atof(Buffer));
}

long Util_Alter_Abteilung (LKF_AUFENTHALT *pAufenthalt, const char *abtdat)
{ /* V8.1: Berechnung des Abteilungsalters */
	int y_auf, y_geb, m_auf, m_geb ,d_auf, d_geb;

	GetDate8 (abtdat, &y_auf, &m_auf, &d_auf);
	GetDate8 (pAufenthalt->GEBDAT, &y_geb, &m_geb, &d_geb);
	
	if (m_auf < m_geb || (m_auf == m_geb && d_auf < d_geb)) return y_auf - y_geb - 1;
	else return y_auf - y_geb;
	
}

long Util_Alter (LKF_AUFENTHALT *pAufenthalt)
{
	int y_ent, y_geb, m_ent, m_geb ,d_ent, d_geb;

	GetDate8 (pAufenthalt->GEBDAT, &y_geb, &m_geb, &d_geb);
    if (pAufenthalt->isAmb)
    {
        GetDate8 (pAufenthalt->AUFDAT, &y_ent, &m_ent, &d_ent);
    }
	else if (pAufenthalt->pScoringContext && (pAufenthalt->ENTART == 'V' || !memcmp (pAufenthalt->ENTDAT, "        ", 8))) /* V12.0 */
	{ /* V11.0 - Altersberechnung bei ENTART = V */
		return pAufenthalt->pScoringContext->cmb_jahr - y_geb;
	}
    else
	{
		GetDate8 (pAufenthalt->ENTDAT, &y_ent, &m_ent, &d_ent);
	}
	
	if (m_ent < m_geb || (m_ent == m_geb && d_ent < d_geb)) return y_ent - y_geb - 1;
	else return y_ent - y_geb;
}

/* function DayOfYear ************************************************
**    Liefert die Information, der wievielte Tag im Jahr der angeg. **
**    Tag ist.                                                      **
*********************************************************************/

int DayOfYear (int year, int month, int day)
{
	static int month_len[12] = {  0,  31,  59,  90, 120, 151, 
		                        181, 212, 243, 273, 304, 334};
	int erg;
	if (month > 0 && month < 13) {
		erg = month_len[month - 1] + day;
		if (month > 2 && IS_SCHALTJAHR (year)) erg++;
		return erg;
	}
	else return 0;

}


/* function GetYearDiffDate8 *****************************************
**    Liefert die Anzahl ganzer Jahre zwischen date1 und date2. Ist **
**    date1 das Geburtsdatum und date2 das aktuelle Datum, so lie-  **
**    fert die Funktion das Alter.                                  **
*********************************************************************/

int GetYearDiffDate8 (const char *date1, const char *date2)
{
	int jahr1, jahr2, monat1, monat2, tag1, tag2;

	GetDate8 (date1, &jahr1, &monat1, &tag1);
	GetDate8 (date2, &jahr2, &monat2, &tag2);

	if (monat2 > monat1 || (monat2 == monat1 && tag2 >= tag1))
		return jahr2 - jahr1;
	else return jahr2 - jahr1 - 1;
}

/** function Util_Diff_tage *******************************
**     Berechnet die Verweildauer des Patienten in einer **
**     bestimmten Abteilung.                             **
**********************************************************/ 

long Util_Diff_Tage(LKF_AUFENTHALT *pAufenthalt, short abtidx)
{
	int y, m, d;
	long date1, date2;

	GetDate8 (pAufenthalt->abt[abtidx].ABTDAT, &y, &m, &d);
	date1 = DatJul (y, m, d);

	if (abtidx == (short) (pAufenthalt->abt_anz - 1))
	{
		if (pAufenthalt->ENTART == 'V')
		{
			char buf[10];
			sprintf (buf, "%04d0101", pAufenthalt->pScoringContext->cmb_jahr + 1);
			date2 = lkf_dat2int (buf);
		}
		else date2 = lkf_dat2int (pAufenthalt->ENTDAT);
	}
	else date2 = lkf_dat2int (pAufenthalt->abt[abtidx + 1].ABTDAT);

	if (date1 && date2 > date1) return date2 - date1;
	else return 0;
}


/* function CopyUCase **************************************************
**     Kopiert den Source- in den Destination-Buffer, wobei Klein-    **
**     buchstaben auf Grobuchstaben konvertiert werden               **
***********************************************************************/

void CopyUCase (char *dest, const char *src, unsigned int len)
{
	unsigned int i;
	for (i = 0; src[i] && i < len; i++) dest[i] = (char) toupper (src[i]);
	while (i < len) dest[i++] = ' ';
}

/* function CopyUCaseLTrim *********************************************
**     analoc CopyUCase, leading blanks werden eliminiert             **
***********************************************************************/

void CopyUCaseLTrim (char *dest, const char *src, unsigned int len)
{
	unsigned int i, j;
	for (j = 0; j < len && src[j] == ' '; j++) ;
	len -= j;
	for (i = 0; src[i+j] && i < len; i++) dest[i] = (char) toupper (src[i+j]);
	if (j) memset (dest + len, ' ', j);
}

void CopyLTrim (char *dest, const char *src, unsigned int len)
{
	unsigned int i, j;
	for (j = 0; j < len && src[j] == ' '; j++) ;
	len -= j;
	for (i = 0; i < len && src[i+j]; i++) dest[i] = src[i+j];
	if (j || i < len) memset (dest + i, ' ', j + len - i);
}

unsigned int Copy (char *dest, const char *src, unsigned int len)
{ /* V15.0 */
	unsigned int i;
	for (i = 0; i < len && src[i]; i++)
		dest[i] = src[i];
	if (i < len)
		memset (dest + i, ' ', len - i);
	return i;
}

void CopyTrimNull (char *dest, const char *src, unsigned int len)
{
	unsigned int i, j;
	for (j = 0; j < len && src[j] == ' '; j++) ;
	len -= j;
	for (i = 0; src[i+j] && i < len; i++) dest[i] = src[i+j];
	for (; i > 0 && dest[i-1] == ' '; i--) ;
	dest[i] = 0;

}


int simLTrim(char *pDes, const char *pSrc, int srclen)
{
	int blanklen = 0;

	while (blanklen < srclen && pSrc[blanklen] == ' ') ++blanklen;

	if (pDes) {
		if (blanklen < srclen) 
			memmove(pDes, pSrc + blanklen, srclen - blanklen);

		if (blanklen) 
			memset(pDes + srclen - blanklen, ' ', blanklen);	/* den rest auf blank setzen*/
	}
	return srclen - blanklen;
}

int simTrim (char *pDes, const char *pSrc, int srclen)
{
	int i, leading_blanks, trailing_blanks, nNull, orig_srclen;

	/* V9.1 */
	nNull = srclen; 
	for (i = srclen; i > 0; i--) if (!pSrc[i-1]) nNull = i - 1;
	orig_srclen = srclen; /* V9.1a */	
	srclen = nNull;

	leading_blanks = 0; 
	while (leading_blanks < srclen && pSrc[leading_blanks] == ' ') leading_blanks++;

	if (leading_blanks == srclen) {
		if (pDes) memset (pDes, ' ', orig_srclen);
		return 0;
	}

	trailing_blanks = 0;
	while (pSrc[srclen - trailing_blanks - 1] == ' ') trailing_blanks++;

	if (pDes) {
		memmove (pDes, pSrc + leading_blanks, srclen - leading_blanks - trailing_blanks);
	
		if (leading_blanks + trailing_blanks)
			memset (pDes + srclen - leading_blanks - trailing_blanks, ' ', leading_blanks + trailing_blanks);
	}

	return srclen - leading_blanks - trailing_blanks;
}

int GetLength (const char *src, int len)
{
	while (len && src[len-1] == ' ') --len;
	return len;
}

int HasBlanks (const char *src, unsigned int len)
{
	unsigned int i;
	for (i = 0; i < len; i++) if (src[i] == ' ') return 1;
	return 0;
}

/* function TrimEqual *************************************************
**     Liefert 1, wenn str in der Lnge len abgesehen von leading    **
**     oder trailing blanks dem nullterminierten String cmp ent-     **
**     spricht - sonst 0.                                            **
**********************************************************************/

int TrimEqual (const char *str, int len, const char *cmp)
{
	int i;
	int cmplen = (int) strlen (cmp);

	/* Ermittlung der tatschlichen Stringlnge (maximal len) */	
	for (i = 0; i < len && str[i] != 0; i++);
	len = i;

	/* Leading Blanks eliminieren */
	while (len && *str == ' ')
	{ --len; ++str; }

	/* Trailing Blanks eliminieren */
	while (len && str[len - 1] == ' ') --len;

	/* Vergleich */
	if (len != cmplen || memcmp (str, cmp, (size_t) cmplen)) return 0;
	else return 1;
}

/*
int UtilICDIndex (const char *icd_text)
{
	return (icd_text[0] - 64) * 100 + (icd_text[1] - '0') * 10 + (icd_text[2] - '0');
}*/

long simCLng(double dNumber)
{
	/* diese Routine soll das dokumentierte Basic verhalten simulieren*/
	/* Basic runded bei werten von genau x.5 zur nachsten GERADEN (?) Zahl*/
	/* zb.: CLng(1.5)=>2  // CLng(2.5)=>2 wird ebenfalls zwei*/
	/* leider verhlt sich basic zur laufzeit nicht so; soda folgendes auftritt*/
	/* ???????????????????????????????????????????????????????????????????????*/
	/* CLng(1996.5) =>1996 jedoch CLng(11/14*2541)=>1997  ????????????????????*/
	/* ???????????????????????????????????????????????????????????????????????*/
double dVorkomma, dNachkomma;
long lVorkomma;
	dNachkomma = modf(dNumber, &dVorkomma);
	lVorkomma = (long) dVorkomma;
	if (fabs(dNachkomma) <= 0.499999) return lVorkomma;
	if (fabs(dNachkomma) >= 0.500001) return (dNumber >= 0.0 ? lVorkomma+1 : lVorkomma-1);
	/* equal 0.5*/
	if ((lVorkomma%2) == 0)/*even number*/ return lVorkomma;
	else                   /*odd  number*/ return (dNumber >= 0.0 ? lVorkomma+1 : lVorkomma-1);
}

int TrimCompare (const char *str, int len, const char *cmp, char krit)
{
	int i;
	for (i = 0; i < len && str[i]; i++) ;
	len = i;

	while (len && *str == ' ')
	{ --len; ++str; }

	while (len && str[len - 1] == ' ') --len;

	while (len && *cmp) {
	
		if (*str > *cmp) {
			if (krit & PAT_KRIT_GREATER) return 1;
			else if ((krit & PAT_KRIT_LESS) || (krit & PAT_KRIT_EQUAL)) return 0;
		}
		else if (*str < *cmp) {
			if ((krit & PAT_KRIT_GREATER) || (krit & PAT_KRIT_EQUAL)) return 0;
			else if (krit & PAT_KRIT_LESS) return 1;
		}
	
		++str; ++cmp; --len;
	}

	while (*cmp == ' ') ++cmp;

	if (len) {
		if (krit & PAT_KRIT_GREATER) return 1;
		return 0;
	}
	else if (*cmp) {
		if (krit & PAT_KRIT_LESS) return 1;
		else return 0;
	}
	else if (krit & PAT_KRIT_EQUAL) return 1;
	else return 0;
}

/* function TrimWildcardEqual *****************************************
**     Wie TrimEqual, nur darf der Compare-String cmp die Wildcard-  **
**     symbole '*' fr beliebig viele (auch 0) Zeichen und '%' fr   **
**     genau ein beliebiges Zeichen enthalten.                       **
**********************************************************************/

static int WildcardEqual (const char *str, int len, const char *cmp)
{
	while (*cmp && len) {
	
		switch (*cmp) {
		
			case PAT_KRIT_WILDCARD_ONE_CHARACTER:

				break;

			case PAT_KRIT_WILDCARD_SOME_CHARACTERS:

				while (*cmp == '*') ++cmp;
				if (!*cmp) return 1;

				while (len) {
					if (TrimWildcardEqual (str, len, cmp)) return 1;
					++str; --len;
				}
				return 0;
			
			default: 
				if (*str != *cmp) return 0;
		
		}
	
		++cmp; ++str; --len;
	}
	
	if (len) return 0;
	
	while (*cmp == '*') ++cmp;
	if (*cmp) return 0;

	return 1;
}

int TrimWildcardEqual (const char *str, int len, const char *cmp)
{
	while (len && *str == ' ')
	{ --len; ++str; }

	while (len && str[len - 1] == ' ') --len;
	return WildcardEqual (str, len, cmp);
}

const char *get_string_ext (const char *src, unsigned int offset)
{ /* V11.1 */
	unsigned int l = (unsigned int) strlen (src);
	if (l <= offset) return 0;
	else return src + offset;
}



int expand_string (const char *src, int len, char *buf)
{ /* V5.1 */
	int l;
	strncpy (buf, src, len);
	buf[len - 1] = 0;
	
	l = (int) strlen (buf);
	while (l > 0 && (buf[l-1]=='\n' || buf[l-1]=='\r')) --l; /* V11.1: \r fuer Unix-kompatible Files */
	
	if (len - l - 1 > 0) memset (buf + l, ' ', len - l - 1);
	return l;
}

/************************************************************/
/* Nachbau der Basicfunktionen								*/
/************************************************************/

static const int daysinmonth[12] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};
static const int daysaccu[12] =    {  0, 31, 59, 90,120,151,181,212,243,273,304,334};

void DatLongToYMD (long lDate, int *pYear, int *pMonth, int *pDay)
{ /* V11.0 - stimmt bis inkl. Jahr 2099 */
	int nYear, nMonth, nDay, n, nSchaltJahr;

	nYear = (int) (lDate / (366*97+365*303));
	lDate -= nYear * (366*97+365*303);
	nYear *= 400;

	nDay = (int) (lDate / (366*24+365*76));
	lDate -= nDay * (366*24+365*76);
	nYear += (nDay*100);

	nDay = (int) (lDate / (365*4+1));
	lDate -= nDay * (365*4+1);
	nYear += 4 * nDay;
	nYear++;

	nSchaltJahr = 0;
	if (lDate >= 3*365)
	{
		nYear += 3;
		lDate -= 3*365;
        nSchaltJahr = 1;
	}
	else if (lDate > 2*365)
	{
		nYear += 2;
		lDate -= 2 * 365;
	}
	else if (lDate > 365)
	{
		nYear++;
		lDate -= 365;
	}
		

	nMonth = 0;

	for (n = 0; !nMonth && n < 12; n++)
	{
		if (n == 1 && nSchaltJahr)
		{
			if (29 > lDate)
			{
				nMonth = 2;
				nDay = (int)(lDate + 1);
			}
			else lDate -= 29;
		}
		else if (daysinmonth[n] > lDate)
		{
			nMonth = n + 1;
			nDay = (int)(lDate + 1);
		}
		else lDate -= daysinmonth[n];
	}

	*pYear = nYear;
	*pMonth = nMonth;
	*pDay = nDay;
}


long DatJul (int year, int month, int day)
{
	
	long value;

	if (year < 0 || month < 1 || month > 12 || day < 1 || day > 31) return -1;

	if (month == 2) {
		if (!(year%4) && (year%100 || !(year%400))) {
			if (day > 29) return -1;
		}
		else if (day > 28) return -1;
	}
	else if (day > daysinmonth[month-1]) return -1;


	value = (year-1) * 365;
	if (month < 3) --year;
	value += (year / 4) - (year/100) + (year/400);
	value += daysaccu[month-1] + day - 1;
	return value;
}



/************************************************************************
** Funktion lkf_dat2int: Konvertiert ein Datum der Form yyyymmdd in    **
**     eine Zahl, wobei aufeinanderfolgende Zahlen aufeinanderfolgende **
**     Tage darstellen.                                                **
************************************************************************/

static long str2int (const char *str, unsigned int len)
{
	long value;
	unsigned int i;
	i = 0;
	value = 0;

	/* Allow leading blanks */
	while (i < len && str[i] == ' ') ++i;
	if (i == len) return -1;

	/* Compute Integer Value */
	while (i < len) {
		if (str[i] < '0' || str[i] > '9') return -1;
		value = value * 10 + (str[i++] - '0');
	}
	return value;
}

long lkf_str2int (const char *str, unsigned int len) {
    return str2int(str, len);
}



long lkf_time2int (const char *tim)
{ /* V8.1 - konvertiert eine Zeitangabe in einen Ganzzahlwert */
	int stunden, minuten;
	stunden = (int) str2int (tim, 2);
	minuten = (int) str2int (tim + 2, 2);

	if (stunden < 0 || stunden > 23 || minuten < 0 || minuten > 59) return 0;
	else return minuten + stunden * 60 + 1;
}


char *lkf_int2dat (long lDat, char *sDatBuf)
{ /* V11.0 */
	int nYear, nMonth, nDay;
	DatLongToYMD (lDat, &nYear, &nMonth, &nDay);
	sprintf (sDatBuf, "%02d.%02d.%04d", nDay, nMonth, nYear);
	return sDatBuf;
}


long lkf_dat2int (const char *dat)
{	
	int year, month, day;

	/* V7.0 - BUGFIX: 0 statt -1 wird im DatJul-Fehlerfall geliefert */
	if (GetDate8 (dat, &year, &month, &day)) {
		long d = DatJul (year, month, day);
		if (d > 0) return d;
		else return 0;
	}
	else return 0;
}

/*
static void test_int2dat () {
    char datbuf[12];
    printf ("%10ld/%s\n", lkf_dat2int("19981231"), lkf_int2dat(lkf_dat2int("19981231"), datbuf));
    printf ("%10ld/%s\n", lkf_dat2int("19990228"), lkf_int2dat(lkf_dat2int("19990228"), datbuf));
    printf ("%10ld/%s\n", lkf_dat2int("19990301"), lkf_int2dat(lkf_dat2int("19990301"), datbuf));
    printf ("%10ld/%s\n", lkf_dat2int("19991231"), lkf_int2dat(lkf_dat2int("19991231"), datbuf));
    printf ("%10ld/%s\n", lkf_dat2int("20000101"), lkf_int2dat(lkf_dat2int("20000101"), datbuf));
    printf ("%10ld/%s\n", lkf_dat2int("20000228"), lkf_int2dat(lkf_dat2int("20000228"), datbuf));
    printf ("%10ld/%s\n", lkf_dat2int("20000229"), lkf_int2dat(lkf_dat2int("20000229"), datbuf));
    printf ("%10ld/%s\n", lkf_dat2int("20000301"), lkf_int2dat(lkf_dat2int("20000301"), datbuf));
    printf ("%10ld/%s\n", lkf_dat2int("20000401"), lkf_int2dat(lkf_dat2int("20000401"), datbuf));
    printf ("%10ld/%s\n", lkf_dat2int("20001201"), lkf_int2dat(lkf_dat2int("20001201"), datbuf));
    printf ("%10ld/%s\n", lkf_dat2int("20010101"), lkf_int2dat(lkf_dat2int("20010101"), datbuf));
    printf ("%10ld/%s\n", lkf_dat2int("20010110"), lkf_int2dat(lkf_dat2int("20010110"), datbuf));
    printf ("%10ld/%s\n", lkf_dat2int("20010228"), lkf_int2dat(lkf_dat2int("20010228"), datbuf));
    printf ("%10ld/%s\n", lkf_dat2int("20010301"), lkf_int2dat(lkf_dat2int("20010301"), datbuf));
    printf ("%10ld/%s\n", lkf_dat2int("20020210"), lkf_int2dat(lkf_dat2int("20020210"), datbuf));
    printf ("%10ld/%s\n", lkf_dat2int("20020310"), lkf_int2dat(lkf_dat2int("20020310"), datbuf));
}
*/


/* function GetDate8 ************************************************
**    berprft das Format des Datums in text (YYYYMMDD) und lie-  **
**    Jahr, Monat und Tag zurck.                                  **
********************************************************************/

int GetDate8 (const char *text, int *year, int *month, int *day)
{
	*year = (int) str2int (text, 4);
	*month = (int) str2int (text + 4, 2);
	*day = (int) str2int (text + 6, 2);

	if (*year < 0 || *month < 0 || *day < 0) return 0;
	else return 1;

}

/* function Util_Datum_Format ********************************************
**    berprft, ob das Datumsformat pat und dieses Datum auch gltig  **
**    ist (und zwischen 1850 und 2200 liegt).                           **
*************************************************************************/

int Util_Datum_Format (const char *datum)
{
	if (lkf_dat2int (datum) > 0) return 0;
	else return 1;
}

/* function SmallerDate8 ********************************************
**    Liefert 1, wenn date1 vor date2 liegt, sonst 0.              **
********************************************************************/

int SmallerDate8 (const char *date1, const char *date2)
{
	long d1, d2;

	d1 = lkf_dat2int (date1);
	d2 = lkf_dat2int (date2);

	if (d1 < d2) return 1;
	else return 0;
}


/****************************************************************************
** lkf_mel_chronodays: Ermittelt, an wie vielen aufeinanderfolgenden Tagen **
**      die gleiche mel in einem Aufenthalt erbracht wurde.                **
****************************************************************************/

static int cmp_long (const void *l1, const void *l2)
{
	return (int)(*(long *)l1 - *(long *)l2);
}

int lkf_mel_chronodays (LKF_AUFENTHALT *pAufenthalt, int mel)
{
	int act_chrono_days, chrono_days;
	long *plMelDat, lMelDat;
	int actmel;
	unsigned int i, count;
	plMelDat = (long *) LKF_CALLOC (sizeof (long), pAufenthalt->mel_anz);

	count = 0;

	for (i = 0; i < pAufenthalt->mel_anz; i++) {
		lkf_GetInt (pAufenthalt->mel[i].MEL, LKF_AUFENTHALT_NEWMEL_LEN, &actmel);
		if (actmel == mel) {
			lMelDat = lkf_dat2int (pAufenthalt->mel[i].MELDAT);
			if (lMelDat) plMelDat[count++] = lMelDat;
		}
	}

	chrono_days = 0;

	if (count) {
		qsort (plMelDat, count, sizeof(long), cmp_long);

		act_chrono_days = 1;
		for (i = 1; i < count; i++) {	
		
			if (plMelDat[i] == plMelDat[i-1]+1) ++act_chrono_days;
			else {
				if (act_chrono_days > chrono_days) chrono_days = act_chrono_days;
				act_chrono_days = 1;
			}
		}
		if (act_chrono_days > chrono_days) chrono_days = act_chrono_days;
	}


	LKF_FREE (plMelDat);
	return chrono_days;
}

int lkf_mel_chronodays_new (LKF_AUFENTHALT *pAufenthalt, int melindex)
{ /* V13.0 - test */
	int act_chrono_days, chrono_days;
	long *plMelDat, lMelDat;
	unsigned int i, count;
	plMelDat = (long *) LKF_CALLOC (sizeof (long), pAufenthalt->mel_anz);

	count = 0;

	for (i = 0; i < pAufenthalt->mel_anz; i++) {
		if (pAufenthalt->mel[i].nMelIndex == melindex) {
			lMelDat = lkf_dat2int (pAufenthalt->mel[i].MELDAT);
			if (lMelDat) plMelDat[count++] = lMelDat;
		}
	}

	chrono_days = 0;

	if (count) {
		qsort (plMelDat, count, sizeof(long), cmp_long);

		act_chrono_days = 1;
		for (i = 1; i < count; i++) {	
		
			if (plMelDat[i] == plMelDat[i-1]+1) ++act_chrono_days;
			else {
				if (act_chrono_days > chrono_days) chrono_days = act_chrono_days;
				act_chrono_days = 1;
			}
		}
		if (act_chrono_days > chrono_days) chrono_days = act_chrono_days;
	}

	LKF_FREE (plMelDat);
	return chrono_days;
}

/********************************************************************************************
** Dynamische Zeilenverwaltung                                                             **
********************************************************************************************/

LKF_LINE_BUFFER *lkf_create_linebuffer (void)
{
	LKF_LINE_BUFFER *pBuffer = (LKF_LINE_BUFFER *) LKF_MALLOC (sizeof(LKF_LINE_BUFFER));
	pBuffer->lBufAlloc = LKF_LINE_BUFFER_BUFALLOC;
	pBuffer->lLineAlloc = LKF_LINE_BUFFER_LINEALLOC;
	pBuffer->pBuf = (char *) LKF_MALLOC (pBuffer->lBufAlloc * sizeof (char));
	pBuffer->plOffsets = (unsigned long *) LKF_MALLOC (pBuffer->lLineAlloc * sizeof(unsigned long));
	pBuffer->lLineCount = 0;
	pBuffer->lBufLen = 0; 
	pBuffer->lBufExtend = LKF_LINE_BUFFER_BUFEXTEND;
	pBuffer->lLineExtend = LKF_LINE_BUFFER_LINEEXTEND;
	return pBuffer;
}

void lkf_drop_linebuffer (LKF_LINE_BUFFER *pBuffer)
{
	if (pBuffer) {
		LKF_FREE (pBuffer->plOffsets);
		LKF_FREE (pBuffer->pBuf);
		LKF_FREE (pBuffer);
	}
}

unsigned int lkf_add_dynamic_line (LKF_LINE_BUFFER *pLineBuffer, const char *sBuf)
{
	const char *sStoredBuffer;
	size_t len = strlen (sBuf);
	while (len > 0 && (sBuf[len-1] == 10 || sBuf[len-1] == 13)) --len;

	if (pLineBuffer->lLineCount == pLineBuffer->lLineAlloc) {
		unsigned long *ppOldBuf = pLineBuffer->plOffsets;

		if (!pLineBuffer->lLineExtend) pLineBuffer->lLineAlloc++;
		else pLineBuffer->lLineAlloc += pLineBuffer->lLineExtend;

		pLineBuffer->plOffsets = (unsigned long *) LKF_MALLOC (pLineBuffer->lLineAlloc * sizeof(unsigned long));
		memcpy (pLineBuffer->plOffsets, ppOldBuf, sizeof (unsigned long) * pLineBuffer->lLineCount);
		LKF_FREE (ppOldBuf);
	}

	if (pLineBuffer->lBufLen + len + 1 >= pLineBuffer->lBufAlloc) {
		
        unsigned long extendBytes;
        unsigned int needed_bytes = (unsigned int) (pLineBuffer->lBufLen + len + 1 - pLineBuffer->lBufAlloc);
		char *pOldBuf = pLineBuffer->pBuf;
		if (needed_bytes < pLineBuffer->lBufExtend) needed_bytes = (unsigned int)(pLineBuffer->lBufExtend);
		
        extendBytes = pLineBuffer->lBufAlloc / 2;
        
        if (needed_bytes < extendBytes) {
            pLineBuffer->lBufAlloc += extendBytes;
        }
        else {
            pLineBuffer->lBufAlloc += (unsigned int) needed_bytes;
        }

		pLineBuffer->pBuf = (char *) LKF_MALLOC (pLineBuffer->lBufAlloc * sizeof (char));
		memcpy (pLineBuffer->pBuf, pOldBuf, sizeof (char) * pLineBuffer->lBufLen);
		LKF_FREE (pOldBuf);
	}

	pLineBuffer->plOffsets[pLineBuffer->lLineCount++] = pLineBuffer->lBufLen;
	sStoredBuffer = pLineBuffer->pBuf + pLineBuffer->lBufLen;
	memcpy (pLineBuffer->pBuf + pLineBuffer->lBufLen, sBuf, len);
	pLineBuffer->pBuf[pLineBuffer->lBufLen + len] = 0;
	pLineBuffer->lBufLen += (unsigned long) len + 1;

	return (unsigned int) pLineBuffer->lLineCount - 1;
}

const char *lkf_get_line (LKF_LINE_BUFFER *pLineBuffer, unsigned int index)
{
	if (index < pLineBuffer->lLineCount) return pLineBuffer->pBuf + pLineBuffer->plOffsets[index];
	else return 0;
}

int lkf_get_linelen (LKF_LINE_BUFFER *pLineBuffer, unsigned int index)
{
	if (index < pLineBuffer->lLineCount - 1) return (int) (pLineBuffer->plOffsets[index + 1] - pLineBuffer->plOffsets[index] - 1);
	else if (index == pLineBuffer->lLineCount - 1) return (int) (pLineBuffer->lBufLen - pLineBuffer->plOffsets[index] - 1);
	else return -1;
}

/********************************************************************************************
** Memory Check Routinen                                                                   **
********************************************************************************************/

#ifdef LKF_MEMORY_LOG

typedef struct
{
	size_t size;
	size_t next_free;
	char *mem;
} LKF_HEAP;

static LKF_HEAP **memblock = 0;
static int memblock_anz = 0;
static int memblock_alloc = 0;

#define OWN_ALLOC 1

#define BEGIN_OFFSET 16
#define END_OFFSET 16
#define DESC_LENGTH 16
#define FILLER_CHAR 67
#define ALLOC_BLOCKSIZE 10000000



typedef struct LKF_MEMBLOCK 
{
	void *address;
	size_t siz;
} LKF_MEMBLOCK;

static LKF_MEMBLOCK *mb = 0;
static unsigned long mb_alloc = 0;
static unsigned long mb_anz = 0;

static int first = 1;
static FILE *fp = 0;

FILE *lkfLogOpenFile (void)
{
	if (!fp) fp = fopen (LKF_LOG_FILENAME, first ? "w" : "a");
	first = 0;
	return fp;
}

void lkfLogFlush (void)
{
	//if (fp) fclose (fp);
	//fp = 0;
}

unsigned int lkf_memory_findpos (void *address)
{
	unsigned long lb, ub, act;
	lb = -1;
	ub = mb_anz;
	while (lb + 1 < ub) {
		act = (lb + ub) / 2;
		if (mb[act].address == address) return act;
		else if (mb[act].address < address) lb = act;
		else ub = act;
	}
	return ub;
}

size_t lkf_memory_add (void *address, size_t siz)
{
	unsigned int pos;
	if (mb_alloc == mb_anz) {
		LKF_MEMBLOCK *n = malloc ((mb_alloc+=100)*sizeof(LKF_MEMBLOCK));
		if (mb) {
			memcpy (n, mb, sizeof(LKF_MEMBLOCK) * mb_anz);
			free (mb);
		}
		mb = n;
	}
	pos = lkf_memory_findpos (address);
	if (pos < mb_anz) memmove (mb + pos + 1, mb + pos, sizeof(LKF_MEMBLOCK) * (mb_anz - pos));
	mb[pos].address = address;
	mb[pos].siz = siz;
	mb_anz++;
	return siz;
}

size_t lkf_memory_delete (void *address)
{
	unsigned int pos;
	pos = lkf_memory_findpos (address);
	if (mb[pos].address != address) {
		fprintf (lkfLogOpenFile (), "MEMORY FREE ERROR: %08X\n", (unsigned int) address);
		lkfLogFlush();
		return 0;
	}
	else {
		size_t siz;
		siz = mb[pos].siz;
		if (pos + 1 < mb_anz) memmove (mb + pos, mb + pos + 1, sizeof(LKF_MEMBLOCK) * (mb_anz - pos - 1));
		--mb_anz;
		return siz;
	}
}

static int open_ptr = 0;
static int linecount = 0;
static long open_memory = 0;


size_t lkf_memory_log (const char *func, void *np, size_t siz, void *free_p)
{
    size_t memsize = 0;
	if (free_p) {
		open_ptr--;
		memsize = lkf_memory_delete (free_p);
		open_memory -= (long) memsize; 
	}
	if (np) {
		open_ptr++;
		memsize = lkf_memory_add (np, siz);
		open_memory += (long) memsize;
	}
    
    if (siz > 1000000) {
        fprintf (lkfLogOpenFile(), "memlog;%d;%s;#%08lX;%d;#%08lX;%d;%ld\n", ++linecount, func, (unsigned long) np, siz, (unsigned long) free_p, open_ptr, open_memory);
    }
    
	lkfLogFlush();
	return memsize;
} 


void free_memory (void *ptr)
{
    unsigned long bytecount;
	unsigned int i,chg1, chg2, off1, off2;
	char *pb, *pe;
	pb = ((char *)ptr) - BEGIN_OFFSET - DESC_LENGTH;
	bytecount = ((long *)pb)[-2];

    if (((long *)pb)[-1] < 0) {
        fprintf (lkfLogOpenFile(), "Memory Freed Twice: Block=%08lX, Size=%ld\n",
                 (unsigned long) ptr, bytecount - BEGIN_OFFSET - END_OFFSET - DESC_LENGTH - sizeof (long));
        return;
    }
    ((long *)pb)[-1] *= -1;
    
	chg1 = chg2 = off1 = off2 = 0;
	for (i = DESC_LENGTH; i < BEGIN_OFFSET + DESC_LENGTH; i++) {
		if (pb[i] != FILLER_CHAR) {
			++chg1;
			if (!off1) off1 = i;
		}
	}
	pe = pb + bytecount - END_OFFSET - 2*sizeof (long);
	for (i = 0; i < END_OFFSET; i++) {
		if (pe[i] != FILLER_CHAR) {
			++chg2;
			if (!off2) off2 = i;
		}
	}
	if (chg1 || chg2) {
		fprintf (lkfLogOpenFile(), "Memory Overwrite detected: Block=%08lX, Size=%ld, Before=%d, After=%d Off1=%d Off2=%d\n",
				 (unsigned long) ptr, bytecount - BEGIN_OFFSET - END_OFFSET - DESC_LENGTH - 2*sizeof (long), chg1, chg2, off1, off2);
	
		if (chg1) {
			fprintf (lkfLogOpenFile(), "B: ");
			for (i = DESC_LENGTH; i < BEGIN_OFFSET + DESC_LENGTH; i++) fprintf (lkfLogOpenFile(), "%02X ", (unsigned char) pb[i]);
			fprintf (lkfLogOpenFile(), "\n");
		}
		if (chg2) {
			fprintf (lkfLogOpenFile(), "E: ");
			for (i = 0; i < END_OFFSET; i++) fprintf (lkfLogOpenFile(), "%02X ", (unsigned char) pe[i]);
			fprintf (lkfLogOpenFile(), "\n");
		}
		lkfLogFlush();
	}
	memset (ptr, FILLER_CHAR, bytecount - BEGIN_OFFSET- END_OFFSET- 2*sizeof (long));
}

void *reserve_memory (size_t bytecount, const char *t)
{
	char *return_memory;
	unsigned int act;
	size_t necessary_bytes;
    long *data;
    static int blockIndex = 0;

    bytecount = (bytecount + 15) / 16 * 16;
            
	necessary_bytes = bytecount + BEGIN_OFFSET + END_OFFSET + DESC_LENGTH + 2*sizeof(long);

	act = memblock_anz - 1;

	if (!memblock || memblock[act]->next_free + necessary_bytes > memblock[act]->size) {
		LKF_HEAP **old_memblock = memblock;
		memblock = (LKF_HEAP **) calloc (++memblock_alloc, sizeof (LKF_HEAP *));
		memblock[memblock_anz++] = (LKF_HEAP *) calloc (1, sizeof (LKF_HEAP));
		act++;
		memblock[act]->next_free = 0;
		memblock[act]->size = ALLOC_BLOCKSIZE;
		memblock[act]->mem = (char *) malloc (ALLOC_BLOCKSIZE);
		memset (memblock[act]->mem, FILLER_CHAR, ALLOC_BLOCKSIZE);
		if (old_memblock) {
			memcpy (memblock, old_memblock, sizeof (LKF_HEAP **) * (memblock_alloc - 1));
			free (old_memblock);
		}
	}
    
	return_memory = memblock[act]->mem + memblock[act]->next_free + BEGIN_OFFSET + DESC_LENGTH + 2*sizeof(long);
    
    data = ((long *) (memblock[act]->mem + memblock[act]->next_free));
    data[0]  = (long) necessary_bytes;
    data[1]  = (long) (++blockIndex);
    
	memset(return_memory - BEGIN_OFFSET - DESC_LENGTH, 0, DESC_LENGTH);
	if (t) {
		strncpy(return_memory - BEGIN_OFFSET - DESC_LENGTH, t, DESC_LENGTH - 1);
	}

    memblock[act]->next_free += necessary_bytes;
	return (void *) return_memory;
}

void realloc_copy (void *dest, void *src)
{
	char *p;
	long old_size, new_size, min_size;

	p = ((char *) src) - BEGIN_OFFSET - DESC_LENGTH;
	old_size = ((long *)p)[-2] - BEGIN_OFFSET - DESC_LENGTH - END_OFFSET - 2*sizeof(long);
	p = ((char *) dest) - BEGIN_OFFSET - DESC_LENGTH;
	new_size = ((long *)p)[-2] - BEGIN_OFFSET - DESC_LENGTH - END_OFFSET - 2*sizeof(long);
	min_size = old_size < new_size ? old_size : new_size;
	memcpy (dest, src, min_size);
}

/********************************************************************************
** call confirm_open_blocks() before checking for open blocks                  **
** report_open_blocks() will then report all memory allocations since that     **
**      call that were not freed.                                              **
********************************************************************************/

_SCORING_EXPORT void lkf_debugmemory_close() {
	if (fp) fclose(fp);
	fp = 0;
}

_SCORING_EXPORT void lkf_debugmemory_write_marker(const char *t) {
	lkf_memory_log(t, 0, 0, 0);
}

_SCORING_EXPORT void lkf_debugmemory_confirm_open_blocks ()
{
    int i;
    long ms;
    long *pos;
    size_t s;
    for (i = 0; i < memblock_anz; i++) {
        
        s = 0;
        while (s < memblock[i]->next_free) {
            ms = *(long*)(memblock[i]->mem + s);
            pos = (long*)(memblock[i]->mem + s + sizeof(long));
            if (*pos) *pos |= 2;
            s += ms;
        }
    }
}

_SCORING_EXPORT void lkf_debugmemory_report_open_blocks()
{
    int i;
    long ms;
    long *pos;
    size_t s;
    for (i = 0; i < memblock_anz; i++) {
        
        s = 0;
        while (s < memblock[i]->next_free) {
            ms = *(long*)(memblock[i]->mem + s);
            pos = (long*)(memblock[i]->mem + s + sizeof(long));
            if (*pos > 0) {

				const char *desc = memblock[i]->mem + s + 2 * sizeof(long);
                fprintf (lkfLogOpenFile(), "Open block: Block=%08lX, Index=%ld, Size=%ld, Usage=%s\n",
                         (unsigned long) (memblock[i]->mem + s + 2*sizeof(long) + BEGIN_OFFSET + DESC_LENGTH), *pos, (ms - BEGIN_OFFSET - END_OFFSET - DESC_LENGTH - 2*sizeof (long)), desc);
                
            }
            s += ms;
        }
    }
    
}

void check_memory (void)
{
    int i, j;
    int bufsize;
    long ms, os;
	char *pa;
	int chg1, off1;
	size_t s;

	for (i = 0; i < memblock_anz; i++) {
	
		s = 0;
		while (s < memblock[i]->next_free) {
			ms = *(long*)(memblock[i]->mem + s);
            os = *(long*)(memblock[i]->mem + s + sizeof(long));
            
            if (os<0) {
                pa = memblock[i]->mem + s + 2*sizeof(long) + BEGIN_OFFSET + DESC_LENGTH;
                chg1 = 0;
                off1 = 0;
                bufsize = (int)(ms - 2 * sizeof(long) - BEGIN_OFFSET - DESC_LENGTH - END_OFFSET);
                for (j = 0; j < bufsize; j++) {
                    if (pa[j]!= FILLER_CHAR) {
                        if (!off1) off1 = j;
                        chg1++;
                    }
                }
                if (chg1) {
                    fprintf (lkfLogOpenFile(), "Memory Usage after Freeing: %08lX, index=%ld, Chg=%d, Offset=%d\n",
                             (unsigned long) pa, os, chg1, off1);
                    lkfLogFlush();
                }
            }
			s += ms;
		}	
	}
}

void *lkf_malloc (size_t siz, const char *t)
{
	void *np;
#ifdef OWN_ALLOC
	np = reserve_memory (siz, t);
#else
	np = malloc (siz);
#endif 
	lkf_memory_log ("malloc", np, siz, 0);
	return np;
}

void *lkf_calloc (size_t siz, size_t nr, const char *s)
{
	void *np;
#ifdef OWN_ALLOC
	np = reserve_memory (siz * nr, s);
	memset (np, 0, siz * nr);
#else
	np = calloc (nr, siz);
#endif
	lkf_memory_log ("calloc", np, siz * nr, 0);
	return np;
}

void *lkf_realloc (void *p, size_t siz)
{
	void *np;
#ifdef OWN_ALLOC
	np = reserve_memory (siz, 0);
	realloc_copy (np, p);
	free_memory (p);
#else
	np = realloc (p, siz);
#endif
	lkf_memory_log ("realloc", np, siz, p);
	return np;
}

void lkf_free (void *p)
{
#ifdef OWN_ALLOC
	free_memory (p);
#else
	free (p);
#endif
	lkf_memory_log ("free", 0, 0, p);
}
#else

_SCORING_EXPORT void lkf_debugmemory_confirm_open_blocks() {

}

_SCORING_EXPORT void lkf_debugmemory_write_marker(const char *t) {
}

_SCORING_EXPORT void lkf_debugmemory_report_open_blocks() {

}

_SCORING_EXPORT void lkf_debugmemory_close() {

}

#endif
