//
//  LKFAufenthalt.swift
//  lkfc
//
//  Created by Alexander Holzer on 07.06.18.
//  Copyright © 2018 Alexander Holzer. All rights reserved.
//

import Foundation
@_implementationOnly import LibLKF

public enum LKFAufenthaltFields : UInt32 {
    case flag = 1
    case kanr = 2
    case aufnr = 3
    case gebdat = 4
    case alter = 5
    case geschl = 6
    case staatb = 7
    case staatw = 8
    case plzw = 9
    case aufart = 10
    case neugeb = 11
    case altersgruppe = 12
    case aufid = 13
    case gesvd = 14
    case entart = 15
    case entdat = 16
    case hd = 17
    case kost = 20
    case gruppe = 21
    case knoten = 22
    case vdu = 23
    case vdo = 24
    case vdmw = 25
    case ptk = 26
    case plk = 27
    case scutk = 28
    case sculk = 29
    case vdzu = 30
    case intzu = 31
    case melzu = 32
    case spez = 33
    case ges = 34
    case ldfpk = 35
    case scuk = 36
    case art = 61
    case id = 103
}

public class LKFAufenthalt {
    
    let ptr: OpaquePointer
    weak public var context: LKFContext?
    public var lastTrace : [String]?
    
    public var computed : Bool = false
        
    init (context: LKFContext, index: UInt32) {
        self.context = context
        self.computed = context.computed
        ptr = LibLKF.lkf_get_aufenthalt(context.ptr!, index)
    }
    
    public var lineCount: UInt32 {
        return intLineCount + mbdsLineCount
    }
    
    public var intLineCount: UInt32 {
        return LibLKF.lkf_auf_getintlineanz(ptr)
    }
    
    public var mbdsLineCount: UInt32 {
        return LibLKF.lkf_auf_getlineanz(ptr)
    }
    
    public func addLine (line: String) -> Bool {

        let firstChar = line.first
        if firstChar == nil || firstChar == "\0" {
            return false
        }
        self.computed = false
        let lineBuffer = (line as NSString).utf8String
        LibLKF.lkf_add_line_auf(ptr, lineBuffer)
        return true
    }
    
    public func mbdsLine (at index: UInt32, lineBuffer: UnsafeMutablePointer<CChar>, identBuffer: UnsafeMutablePointer<CChar>) -> String {
        LibLKF.lkf_auf_getLineAndIdent(ptr, index, lineBuffer, identBuffer)
        let s = String(cString: lineBuffer)
        return s
    }
    
    public func lineWith (ident: String, lineBuffer: UnsafeMutablePointer<CChar>) -> String? {
        if LibLKF.lkf_auf_getLineWithIdent(ptr, ident.cString(using: .utf8), lineBuffer) >= 0 {
            return String(cString: lineBuffer)
        }
        return nil
    }
    
    public func lineWith (ident: String) -> String? {
        let lineBuffer = UnsafeMutablePointer<CChar>.allocate(capacity:1024)
        if let s = lineWith (ident: ident, lineBuffer: lineBuffer) {
            lineBuffer.deallocate()
            return s
        }
        lineBuffer.deallocate()
        return nil
    }

    public func intLine (at index: UInt32, lineBuffer: UnsafeMutablePointer<CChar>, identBuffer: UnsafeMutablePointer<CChar>) -> String {
        LibLKF.lkf_auf_getintLineAndIdent(ptr, index, lineBuffer, identBuffer)
        return String(cString: lineBuffer)
    }

    public func intLine (ident: String, lineBuffer: UnsafeMutablePointer<CChar>) -> String? {
        LibLKF.lkf_auf_getLineWithIdent(ptr, ident.cString(using: .utf8), lineBuffer)
        let s = String(cString: lineBuffer)
        return s
    }
    
    public func intLine (ident: String) -> String? {
        let lineBuffer = UnsafeMutablePointer<CChar>.allocate(capacity:1024)
        if let s = intLine(ident: ident, lineBuffer: lineBuffer) {
            lineBuffer.deallocate()
            return s
        }
        lineBuffer.deallocate()
        return nil
    }
    
    public func replaceMbdsLine (with line: String, ident: String) -> Bool {
        self.computed = false
        return LibLKF.lkf_auf_replaceLine(ptr, line.cString(using: .utf8), ident.cString(using: .utf8)) == 0
    }
    
    public func deleteMbdsLine (ident: String) -> Bool {
        self.computed = false
        return LibLKF.lkf_delete_xline(ptr, ident.cString(using: .utf8)) == 0
    }
    
    public func replaceIntLine (with line: String, ident: String) -> Bool {
        self.computed = false
        return LibLKF.lkf_auf_replaceIntLine(ptr, line.cString(using: .utf8), ident.cString(using: .utf8)) == 0
    }
    
    public func deleteIntLine (ident: String) -> Bool {
        self.computed = false
        return LibLKF.lkf_auf_deleteIntLine(ptr, ident.cString(using: .utf8)) == 0
    }

    
    public func getOutput(mbds: Bool = true, intLines: Bool = true) -> [String] {
        
        var outputStrings = [String]()
        let lineBuffer = UnsafeMutablePointer<CChar>.allocate(capacity:1024)

        if mbds {
            let count = LibLKF.lkf_auf_getlineanz(ptr)
            outputStrings.reserveCapacity(Int(count))
            for i in 0..<count {
                LibLKF.lkf_auf_getline(ptr, i, lineBuffer)
                outputStrings.append(String(cString:lineBuffer))
            }
        }
        
        if intLines {
            let count = LibLKF.lkf_auf_getintlineanz(ptr)
            for i in 0..<count {
                LibLKF.lkf_auf_getintline(ptr, i, lineBuffer)
                outputStrings.append(String(cString:lineBuffer))
            }
        }
        

        lineBuffer.deallocate()
        return outputStrings
    }
    
    public func getResultLine () -> String {
        let lineBuffer = UnsafeMutablePointer<CChar>.allocate(capacity:1024)
        LibLKF.lkf_auf_line_scoring(ptr, lineBuffer)
        let s = String(cString: lineBuffer)
        lineBuffer.deallocate()
        return s
    }
    
    public func getField (fieldNum: UInt32) -> String {
        let lineBuffer = UnsafeMutablePointer<CChar>.allocate(capacity:512)
        LibLKF.lkf_auf_get_fieldvalue(ptr, fieldNum, lineBuffer, 512)
        let s = String(cString: lineBuffer)
        lineBuffer.deallocate()
        return s
    }
    
    public func getResultField (fieldNum: UInt32) -> Int {
        return LibLKF.lkf_auf_get_scorvalue(ptr, fieldNum)
    }
    
    public func getField (_ field: LKFAufenthaltFields) -> String {
        let lineBuffer = UnsafeMutablePointer<CChar>.allocate(capacity:512)
        LibLKF.lkf_auf_get_fieldvalue(ptr, field.rawValue, lineBuffer, 512)
        let s = String(cString: lineBuffer)
        lineBuffer.deallocate()
        return s
    }
    
    public func getResultField (_ field:LKFAufenthaltFields) -> Int {
        return LibLKF.lkf_auf_get_scorvalue(ptr, field.rawValue)
    }
    
    public var points : Int {
        return getResultField(fieldNum: 44)
    }
    
    public var errorCount : Int {
        var nextError = LibLKF.lkf_auf_firsterror(ptr)
        var ec = 0
        while nextError != nil {
            ec += 1
            nextError = LibLKF.lkf_auf_nexterror (nextError)
        }
        return ec
    }
    
    public var warningCount : Int {
        return 0
    }
    
    public var infoCount : Int {
        return 0
    }
    
    public var hd : String {
        return getField(fieldNum: 17)
    }
    
    public var aufId: String {
        return getField(fieldNum: 103)
    }
    
    public var gebDat: Date? {
        let s = getField(fieldNum: 4)
        return LKFConvert.toDate (string8:s)
    }

    public var staatb: String {
        return getField(.staatb)
    }

    public var staatw: String {
        return getField(.staatw)
    }

    public var plzw: String {
        return getField(.plzw)
    }
    
    public var aufart : String {
        return getField(.aufart)
    }
    
    public var entart : String {
        return getField(.entart)
    }

    
    public var aufDat: Date? {
        let s = getField(fieldNum: 40)
        return LKFConvert.toDate (string8:s)
    }
    
    public var entDat: Date? {
        let s = getField(.entdat)
        return LKFConvert.toDate(string8: s)
    }
    
    public var geschl: String {
        return getField(fieldNum: 6)
    }
    
    public func getTestOutput() -> [String] {
        return []
    }
    
    public var abtCount : UInt32 {
        return LibLKF.lkf_auf_get_abtanz(ptr)
    }
    
    public var melCount : UInt32 {
        return LibLKF.lkf_auf_get_melanz(ptr)
    }
        
    public var diagCount : UInt32 {
        return LibLKF.lkf_auf_get_diaganz(ptr)
    }
    
    public var mainDiagCount : UInt32 {
        return LibLKF.lkf_auf_get_hdiaganz(ptr)
    }
    
    public func melAt (index: UInt32) -> LKFAufenthaltMEL {
        return LKFAufenthaltMEL(aufenthalt: self, index: index)
    }
    
    public func diagAt (index: UInt32) -> LKFAufenthaltDiag {
        return LKFAufenthaltDiag(aufenthalt: self, index: index)
    }
    
    public func abtAt (index: UInt32) -> LKFAufenthaltAbt {
        return LKFAufenthaltAbt(aufenthalt: self, index: index)
    }
    
    public func getResult() -> LKFAufenthaltResult {
        return LKFAufenthaltResult(aufenthalt: self)
    }
    
    public var isAmbulant : Bool {
        return LibLKF.lkf_auf_isAmb(ptr) != 0;
    }

    public var isAmbulantTKL : Bool {
        return LibLKF.lkf_auf_isAmb(ptr) == 2;
    }
    
    public var key : String {
        let lineBuffer = UnsafeMutablePointer<CChar>.allocate(capacity:512)
        LibLKF.lkf_auf_get_ident(ptr, lineBuffer)
        let s = String(cString: lineBuffer)
        lineBuffer.deallocate()
        return s
    }

    public var id : String {
        return getField(.id)
    }

    
    public var gesvd : UInt32 {
        let s = getField(fieldNum: 14)
        if let v = UInt32(s) {
            return v
        }
        return 9999
    }
    
    public var gruppe : String {
        return getField(fieldNum: 21)
    }
    
    public var knoten : String {
        return getField(fieldNum: 22)
    }
    
    public var gruppe_knoten : String? {
        let gruppe = self.gruppe
        if gruppe.count > 0 {
            let knoten = self.knoten
            if knoten.count > 0  && knoten.first! != " " {
                return "\(gruppe)/\(knoten)"
            }
            return gruppe
        }
        return nil
    }
    
    public func getIntResults() -> [LKFIntAbtResult?] {
        
        var arr = [LKFIntAbtResult?]()
        
        for i in 0..<abtCount {
            arr.append(LKFIntAbtResult(aufenthalt: self, index: i))
        }
        return arr
    }
    
    public func getErrors(format:LKFErrorFormat) -> [String] {
        var errorStrings = [String]()
        if let context = context {
            let lineBuffer = UnsafeMutablePointer<CChar>.allocate(capacity:1024)

            var err = LibLKF.lkf_auf_getSatzartError(ptr)
            while (err != nil) {
                _ = LibLKF.lkf_getSatzartErrorFromFormat (context.ptr, err, format.ptr, 1024, lineBuffer)
                errorStrings.append(String(cString: lineBuffer))
                err = LibLKF.lkf_auf_getNextSatzartError(err)
            }
            lineBuffer.deallocate()
        }
        return errorStrings
    }
    
    private static var defaultErrorFormat = LKFErrorFormat(fields: [.code,.aufnr,.abtfcent,.feld_lang,.text])
    
    public func getErrors() -> [String] {
        return getErrors(format:LKFAufenthalt.defaultErrorFormat)
    }
    
    public func findFirstMel(with melKey: Int64) -> LKFAufenthaltMEL? {
        let index = LibLKF.lkf_auf_findFirstMelIndex(ptr, melKey)
        if index >= 0 {
            return LKFAufenthaltMEL(aufenthalt: self, index: UInt32(index))
        }
        return nil
    }
    
    public func findFirstDiag(with diagKey: Int64) -> LKFAufenthaltDiag? {
        let index = LibLKF.lkf_auf_findFirstDiagIndex(ptr, diagKey)
        if index >= 0 {
            return LKFAufenthaltDiag(aufenthalt: self, index: UInt32(index))
        }
        return nil
    }
    
    public func findFirstAbt (abtfc: String) -> LKFAufenthaltAbt? {
        return nil
    }
            
    public func createTrace () -> [String] {
        if let context = context {
            
            let traceOptions = LKFContextTraceOptions(mbdsLevel: 2, kaskoreLevel: 2, intscorLevel: 2, aufnr: aufId)
            lastTrace = []
            context.tracingStart(options: traceOptions) { s in
                self.lastTrace!.append(s)
            }
            recompute()
            context.tracingEnd()
            return self.lastTrace!
        }
        return []
    }
     
    public func createScoringDetails () -> LKFScoringDetails? {

        guard let context = context else {
            return nil
        }

        guard self.errorCount == 0 else {
            return nil
        }
        
        let traceOptions = LKFContextTraceOptions(mbdsLevel: 2, kaskoreLevel: 2, intscorLevel: 2, aufnr: aufId)
        lastTrace = []
        
        let originalFlag = context.flags
        context.flags = (originalFlag | 128)
        
        context.tracingStart(options: traceOptions) { s in
            self.lastTrace!.append(s)
        }
        recompute()
        context.tracingEnd()
        context.flags = originalFlag

        return LKFScoringDetails(aufenthalt: self)
    }
        
    public func trace () -> String {
        let output = createTrace()
        var result : String = ""
        for s in output {
            result.append(s)
            result.append("\n")
        }
        
        if let scoringDetails = createScoringDetails() {
            result.append("\nSCORING-Details:\n")
            for s in scoringDetails.createTrace() {
                result.append(s)
                result.append("\n")
            }
        }
        return result
    }
    
    public func recompute() {
        LibLKF.lkf_recompute_aufenthalt(ptr)
        self.computed = true
    }
    
    public func getDiags() -> [LKFAufenthaltDiag]? {
        let count = diagCount
        guard count > 0 else {
            return nil
        }
        var arr = [LKFAufenthaltDiag]()
        for i in 0..<count {
            arr.append (diagAt(index: i))
        }
        return arr
    }
    
    public func getMels() -> [LKFAufenthaltMEL]? {
        let count = melCount
        guard count > 0 else {
            return nil
        }
        var arr = [LKFAufenthaltMEL]()
        for i in 0..<count {
            arr.append (melAt(index: i))
        }
        return arr
    }
    
    public func getAbts() -> [LKFAufenthaltAbt]? {
        let count = abtCount
        guard count > 0 else {
            return nil
        }
        var arr = [LKFAufenthaltAbt]()
        for i in 0..<count {
            arr.append (abtAt(index: i))
        }
        return arr
    }
    
    public var optimizable : Bool {
        return LibLKF.lkf_auf_is_optimizable(ptr) != 0
    }
    
    public var optPercentage : Double {
        return LibLKF.lkf_auf_get_opt_percentage(ptr)
    }
}
