//
//  LKFMetaDataDiag.swift
//  LKF
//
//  Created by Alexander Holzer on 14.01.22.
//  Copyright © 2022 Alexander Holzer. All rights reserved.
//

import Foundation
@_implementationOnly import LibLKF

public class LKFMetaDataDiag {
    
    public enum Fields : UInt32 {
        case Diag =  1
        case Group = 4
        case Text = 20
    }
    
    let context : LKFContext
    public let diagIndex: UInt32
    
    public lazy var group : LKFMetaDataGroup? = { () -> LKFMetaDataGroup? in
        let g = GroupName
        if g.count > 0 {
            return LKFMetaDataGroup (context: context, group: g)
        }
        return nil
    }()
    
    public init (context: LKFContext, diagIndex: UInt32) {
        self.context = context
        self.diagIndex = diagIndex
    }
    
    public func value (for field:LKFMetaDataDiag.Fields) -> String {
        if let ptr = context.ptr {
            let lineBuffer = UnsafeMutablePointer<CChar>.allocate(capacity:256)
            LibLKF.lkf_stamm_getDiagField (ptr, diagIndex, field.rawValue, lineBuffer, 256)
            let r = String(cString: lineBuffer).trimmingCharacters(in: .whitespaces)
            lineBuffer.deallocate()
            return r
        }
        return ""
     }
    
    public func value (for field:LKFMetaDataDiag.Fields, encoding: String.Encoding) -> String {
        if let ptr = context.ptr {
            let lineBuffer = UnsafeMutablePointer<CChar>.allocate(capacity:256)
            LibLKF.lkf_stamm_getDiagField (ptr, diagIndex, field.rawValue, lineBuffer, 256)
            if let r = String(cString: lineBuffer, encoding: encoding) {
                lineBuffer.deallocate()
                return r.trimmingCharacters(in: .whitespaces)
            }
        }
        return ""
     }

    
    public var Diag : String {
        return value (for: .Diag)
    }
    
    public var GroupName : String {
        return value (for: .Group)
    }

    public var Text: String {
        return value (for: .Text, encoding: .ascii)
    }
}
