
#include "Stdafx.h"

extern "C" {
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "lkf5defs.h"
}

using namespace System::IO;

#include "LKFController.h"
#include "LKFContext.h"
#include "LKFAufenthalt.h"
#include "LKFException.h"
#include "LKFIntContext.h"
#include "LKFStatContext.h"
#include "LKFNeoPaedContext.h"

namespace lkfxdok {
	namespace c {

		LKFContext::LKFContext()
		{
			_contextPtr = lkf_alloc_context(0, "", 0);
		}

		LKFContext::~LKFContext() {
			this->!LKFContext();
		}

		LKFContext::!LKFContext() {
			if (_contextPtr != nullptr) {
				lkf_free_context((LKF_CONTEXT *)_contextPtr);
				_contextPtr = nullptr;
			}
			
		}

		void LKFContext::Debug_Free() {
			if (_contextPtr != nullptr) {
				lkf_free_context((LKF_CONTEXT *)_contextPtr);
				_contextPtr = nullptr;
			}
		}

		int LKFContext::Flags::get (void) {
			return _flags;
		}
		
		void LKFContext::Flags::set(int flags) {
			_flags = flags;
			lkf_set_contextflags((LKF_CONTEXT *)_contextPtr, flags);
		}

		int LKFContext::Modelljahr::get(void) {
			return _modelljahr;
		}

		void LKFContext::Modelljahr::set(int modelljahr) {
			char errorBuffer[512];
			_modelljahr = modelljahr;

			char* path = (char*)Marshal::StringToHGlobalAnsi(LKFController::DataPath).ToPointer();
			int res = lkf_set_modelljahr((LKF_CONTEXT *)_contextPtr, modelljahr, path, errorBuffer);
			if (res) {
				Marshal::FreeHGlobal((IntPtr)path);
				LKFException ^ex = gcnew LKFException(res, gcnew String(errorBuffer));
				throw ex;
			}
			Marshal::FreeHGlobal((IntPtr)path);
		}

		int LKFContext::Abrechnungsjahr::get ()
		{
			return _abrechnungsjahr;
		}
		
		void LKFContext::Abrechnungsjahr::set(int abrechnungsjahr) {
			_abrechnungsjahr = abrechnungsjahr;
			lkf_set_abrechnungsjahr((LKF_CONTEXT *)_contextPtr, abrechnungsjahr);
		}
		

		void LKFContext::SetAbrechnungsPeriode(int jahr, int period_val, int lfdnr, LKFContextPeriode type, int flags) {
			_abrechnungsjahr = jahr;
			lkf_set_contextparameters((LKF_CONTEXT *)_contextPtr, jahr, period_val, lfdnr, (char)type, flags);
		}

		void LKFContext::EnableTrace(int facility, int level) {
			lkf_set_debug_tracefile((LKF_CONTEXT *)_contextPtr, 0, 0, 0);
			lkf_trace_setLevel((LKF_CONTEXT *)_contextPtr, facility, level);
		}

		void LKFContext::SetTraceAufenthalt(String ^patid) {
			if (patid == nullptr)
				lkf_trace_setAufenthalt((LKF_CONTEXT *)_contextPtr, 0);
			else {
				char* traceAufenthalt = (char*)Marshal::StringToHGlobalAnsi(patid).ToPointer();
				lkf_trace_setAufenthalt((LKF_CONTEXT *)_contextPtr, traceAufenthalt);
				Marshal::FreeHGlobal((IntPtr)traceAufenthalt);
			}
		}

		void LKFContext::EnableDebugTrace() {
			EnableTrace(0, 2);
		}

		unsigned int LKFContext::TraceLineCount::get()
		{
			return lkf_trace_getLineCount((LKF_CONTEXT *)_contextPtr);
		}

		String ^LKFContext::GetTraceLine(unsigned int index) {
			const char *line = lkf_trace_getLine((LKF_CONTEXT *)_contextPtr, index);
			if (line)
				return gcnew String(line);
			return nullptr;
		}

		LKFContextWithData::LKFContextWithData() : LKFContext()
		{
			_lastImportCharCount = 0;
			_lastImportErrorCount = 0;
			_lastImportLineCount = 0;
		}

		unsigned int LKFContextWithData::AufenthaltCount::get(void) {
			return lkf_get_aufenthalt_anz((LKF_CONTEXT *)_contextPtr);
		}

		unsigned int LKFContextWithData::AdditionalLines::get(void) {
			return lkf_getaddlineanz((LKF_CONTEXT *)_contextPtr);
		}

		String ^LKFContextWithData::GetAdditionalLine(unsigned int index) {
			char buf[2048];
			if (!lkf_getaddline((LKF_CONTEXT *)_contextPtr, index, buf))
				return gcnew String(buf);
			return nullptr;
		}

		void LKFContextWithData::PlausCheck() {
			lkf_check((LKF_CONTEXT *)_contextPtr, 0);
		}

		void LKFContextWithData::Scoring() {
			lkf_context_scoring((LKF_CONTEXT *)_contextPtr, 0);
		}

		LKFIntContext ^LKFContextWithData::IntScoring() {
			lkf_int_scoring((LKF_CONTEXT *)_contextPtr);
			return gcnew LKFIntContext(this);
		}

		LKFStatContext ^LKFContextWithData::Stat() {
			lkf_stat((LKF_CONTEXT *)_contextPtr);
			return gcnew LKFStatContext(this);
		}

		LKFNeoPaedContext ^LKFContextWithData::NeoPaed() {
			lkf_stat((LKF_CONTEXT *)_contextPtr);
			return gcnew LKFNeoPaedContext(this);
		}

		LKFAufenthalt ^LKFContextWithData::GetAufenthalt(unsigned int index)
		{
			LKF_AUFENTHALT *aufPtr = lkf_get_aufenthalt((LKF_CONTEXT *)_contextPtr, index);
			if (aufPtr != nullptr)
				return gcnew LKFAufenthalt(this, (void*)aufPtr);
			return nullptr;
		}

		void LKFContextWithData::AddLine(String ^line) 
		{
			char *cline = (char*)(Marshal::StringToHGlobalAnsi(line)).ToPointer();
			lkf_add_line((LKF_CONTEXT *)_contextPtr, cline);
			Marshal::FreeHGlobal((IntPtr)cline);
		};

		void LKFContextWithData::AddBuffer(array<unsigned char> ^buffer)
		{
			int charCount, errorCount;
			char *cBuffer = new char[buffer->Length + 1];
			Marshal::Copy(buffer, 0, IntPtr(cBuffer), buffer->Length);
			cBuffer[buffer->Length] = 0;
			_lastImportLineCount = lkf_add_lines2((LKF_CONTEXT *)_contextPtr, cBuffer, &charCount, &errorCount);
			delete cBuffer;
			_lastImportCharCount = charCount;
			_lastImportErrorCount = errorCount;
		}

		void LKFContextWithData::LoadFile(String ^fileName) 
		{
			StreamReader ^r = gcnew StreamReader(fileName);
			while (!r->EndOfStream) {
				AddLine(r->ReadLine());
			}
			r->Close();
		}

		void LKFContextWithData::WriteFile(LKFContextFileType fileType, String^ outputFileName) {
			StreamWriter^ f = gcnew StreamWriter(outputFileName);
			WriteFile(fileType, f);
			f->Close();
		}
		
		void LKFContextWithData::WriteResultFile(StreamWriter^ f) {
			unsigned int addlineCount, aufenthaltCount, aufenthaltLineCount, i, j, k;
			char line[2048];
			LKF_AUFENTHALT* pAufenthalt;
			String ^lastAddLineAnz;

			addlineCount = lkf_getaddlineanz((LKF_CONTEXT*)_contextPtr);
			for (i = 0; i < addlineCount; i++) {
				lkf_getaddline((LKF_CONTEXT*)_contextPtr, i, line);
				lastAddLineAnz = gcnew String(line);
				if (lastAddLineAnz->StartsWith("K01")) {
					f->WriteLine(lastAddLineAnz);
				}
				else {
					break;
				}
			}

			aufenthaltCount = lkf_get_aufenthalt_anz((LKF_CONTEXT*)_contextPtr);

			for (j = 0; j < aufenthaltCount; j++) {
				pAufenthalt = lkf_get_aufenthalt((LKF_CONTEXT*)_contextPtr, j);
				aufenthaltLineCount = lkf_auf_getlineanz(pAufenthalt);
				for (k = 0; k < aufenthaltLineCount; k++) {
					lkf_auf_getline(pAufenthalt, k, line);
					f->WriteLine(gcnew String(line));
				}
			}

			if (i < addlineCount) {
				f->WriteLine(lastAddLineAnz);
				for (; i < addlineCount; i++) {
					lkf_getaddline((LKF_CONTEXT*)_contextPtr, i, line);
					f->WriteLine(gcnew String(line));
				}
			}
		}

		void LKFContextWithData::WriteFile(LKFContextFileType fileType, StreamWriter^ f) {
			unsigned int count, i;
			char line[2048];

			if (fileType == LKFContextFileType::Stat) {
				Stat()->WriteFile(f);
				return;
			}
			else if (fileType == LKFContextFileType::Result) {
				WriteResultFile(f);
				return;
			}

			switch (fileType) {

				case LKFContextFileType::Error:
					count = lkf_setErrorFilter((LKF_CONTEXT *)_contextPtr, LKF_ERRORFILTER_ALLERRORS);
					lkf_getSatzartErrorHeadline((LKF_CONTEXT *)_contextPtr, LKF_FAC_ALL, 2048, line);
					f->WriteLine(gcnew String(line));
					for (i = 0; i < count; i++) {
						lkf_getSatzartError((LKF_CONTEXT *)_contextPtr, i, 2048, line);
						f->WriteLine(gcnew String(line));
					}
					break;
				case LKFContextFileType::Warning:
					count = lkf_setErrorFilter((LKF_CONTEXT *)_contextPtr, LKF_ERRORFILTER_ALLWARNINGS);
					lkf_getSatzartErrorHeadline((LKF_CONTEXT *)_contextPtr, LKF_FAC_ALL, 2048, line);
					f->WriteLine(gcnew String(line));
					for (i = 0; i < count; i++) {
						lkf_getSatzartError((LKF_CONTEXT *)_contextPtr, i, 2048, line);
						f->WriteLine(gcnew String(line));
					}
					break;

				case LKFContextFileType::Info:
					count = lkf_setErrorFilter((LKF_CONTEXT *)_contextPtr, LKF_ERRORFILTER_ALLINFOS);
					lkf_getSatzartErrorHeadline((LKF_CONTEXT *)_contextPtr, LKF_FAC_ALL, 2048, line);
					f->WriteLine(gcnew String(line));
					for (i = 0; i < count; i++) {
						lkf_getSatzartError((LKF_CONTEXT *)_contextPtr, i, 2048, line);
						f->WriteLine(gcnew String(line));
					}
					break;

				default:
					break;
			}
		}

		void LKFContextWithData::SetErrorFormat(unsigned int facility, array<int> ^fields) {
			if (fields == nullptr) {
				lkf_setErrorFormat((LKF_CONTEXT *)_contextPtr, facility, 0, 0);
			}
			else {
				unsigned int *fieldIndices = new unsigned int[fields->Length];
				Marshal::Copy(fields, 0, IntPtr(fieldIndices), fields->Length);
				lkf_setErrorFormat((LKF_CONTEXT *)_contextPtr, facility, fields->Length, fieldIndices);
				delete fieldIndices;
			}
		}

		unsigned int LKFContextWithData::SetErrorFilter(unsigned int filter) {
			return lkf_setErrorFilter((LKF_CONTEXT *)_contextPtr, filter);
		}

		String  ^LKFContextWithData::GetErrorHeadLine(unsigned int errorIndex) {
			char line[2048];
			lkf_getSatzartErrorHeadline((LKF_CONTEXT *)_contextPtr, LKF_FAC_ALL, 2048, line);
			return gcnew String(line);
		}


		String  ^LKFContextWithData::GetError(unsigned int index) {
			char line[2048];
			lkf_getSatzartError((LKF_CONTEXT *)_contextPtr, index, 2048, line);
			return gcnew String(line);
		}

		String  ^LKFContextWithData::GetErrorField(unsigned int index, unsigned int errorField) {
			char line[512];
			lkf_getSatzartErrorField ((LKF_CONTEXT *) _contextPtr, index, errorField, line, 512);
			return gcnew String(line);
		}

		unsigned int LKFContextWithData::ErrorCount::get(void) {
			return lkf_context_getSatzartErrorAnz((LKF_CONTEXT *)_contextPtr, 12);
		}

		unsigned int LKFContextWithData::WarningCount::get(void) {
			return lkf_context_getSatzartErrorAnz((LKF_CONTEXT *)_contextPtr, 2);
		}

		unsigned int LKFContextWithData::AufenthaltErrorCount::get(void) {
			return lkf_context_GetErrorAnz((LKF_CONTEXT *)_contextPtr, 0, 0);
		}

		unsigned int LKFContextWithData::AufenthaltWarningCount::get(void) {
			return lkf_context_GetWarningAnz((LKF_CONTEXT *)_contextPtr, 0, 0);
		}


		unsigned int LKFContextWithData::InfoCount::get(void) {
			return lkf_context_GetHinweisAnz((LKF_CONTEXT *)_contextPtr, 0, 0);
		}

		unsigned int LKFContextWithData::AcceptedErrorCount::get(void) {
			int acceptedCount;
			lkf_context_GetErrorAnz((LKF_CONTEXT *)_contextPtr, 0, &acceptedCount);
			return (unsigned int) acceptedCount;
		}

		unsigned int LKFContextWithData::AcceptedWarningCount::get(void) {
			int acceptedCount;
			lkf_context_GetWarningAnz((LKF_CONTEXT *)_contextPtr, 0, &acceptedCount);
			return (unsigned int)acceptedCount;
		}

		unsigned int LKFContextWithData::AcceptedInfoCount::get(void) {
			int acceptedCount;
			lkf_context_GetHinweisAnz((LKF_CONTEXT *)_contextPtr, 0, &acceptedCount);
			return (unsigned int)acceptedCount;
		}

		int LKFContextWithData::GetAmbCounter(unsigned int ambIndex) {
			return lkf_get_amb_counter((LKF_CONTEXT *)_contextPtr, ambIndex);
		}

		int LKFContextWithData::LastImportCharCount::get(void) {
			return _lastImportCharCount;
		}

		int LKFContextWithData::LastImportLineCount::get(void) {
			return _lastImportLineCount;
		}

		int LKFContextWithData::LastImportErrorCount::get(void) {
			return _lastImportErrorCount;
		}

		int LKFContextWithData::GetContextValue(int contextvalue) {
			return lkf_get_contextvalue((LKF_CONTEXT *)_contextPtr, contextvalue);
		}

		double LKFContextWithData::GetContextScoringValue(unsigned int contextvalue) {
			return lkf_get_context_scorvalue((LKF_CONTEXT*)_contextPtr, 0, contextvalue);
		}

		// double GetContextScoringValue(int contextvalue);


		LKFContextForStat::LKFContextForStat() : LKFContextWithData()
		{
		}

		void LKFContextForStat::Modelljahr::set(int modelljahr) {
		}

		void LKFContextForStat::SetAbrechnungsPeriode(int jahr, int period_val, int lfdnr, LKFContextPeriode type, int flags) {
			Abrechnungsjahr = jahr;
			lkf_set_contextparameters((LKF_CONTEXT *)_contextPtr, jahr, period_val, lfdnr, (char)type, flags);
		}

		void LKFContextForStat::Abrechnungsjahr::set(int abrechnungsjahr) {
			_abrechnungsjahr = abrechnungsjahr;
			char errorBuffer[512];

			char* path = (char*)Marshal::StringToHGlobalAnsi(LKFController::DataPath).ToPointer();
			int res = lkf_set_statjahr((LKF_CONTEXT *)_contextPtr, abrechnungsjahr, path, errorBuffer);
			if (res) {
				Marshal::FreeHGlobal((IntPtr)path);
				LKFException ^ex = gcnew LKFException(res, gcnew String(errorBuffer));
				throw ex;
			}
			Marshal::FreeHGlobal((IntPtr)path);
			_modelljahr = lkf_context_getModellJahr((LKF_CONTEXT *)_contextPtr);
			lkf_set_abrechnungsjahr((LKF_CONTEXT *)_contextPtr, abrechnungsjahr);
		}
	}
}

