﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using lkfxdok.c;

namespace test.C
{
    public class FileTest
    {
        int _modellJahr;
        int _abrechJahr;
        string _fileName;

        LKFContextWithData _context;

        public FileTest (int modell, int abr, string fileName)
        {
            _modellJahr = modell;
            _abrechJahr = abr;
            _fileName = fileName;
        }

        public void RunStat()
        {
            LKFController.DataPath = "C:\\lkf\\";
            _context = new LKFContextForStat();

            try
            {
                _context.Flags = (int) LKFContextFlags._Optimizer;
                _context.Abrechnungsjahr = _abrechJahr;
            }
            catch (LKFException ex)
            {
                Console.WriteLine(ex.Message);
                return;
            }
            _context.LoadFile(_fileName);
            uint aufenthaltCount = _context.AufenthaltCount;
            Console.WriteLine("Aufenthalte: {0}", aufenthaltCount);
            _context.PlausCheck();
            _context.Scoring();
            _context.WriteFile(LKFContextFileType.Stat, "C:\\LKF\\out\\stat.txt");
            var aufenthalt = _context.GetAufenthalt(0);
            Console.WriteLine(String.Format("{0} Belagstage.", aufenthalt.GESBD));
        }

        public void RunNP ()
        {
            LKFController.DataPath = "C:\\lkf\\";

            _context = new LKFContextWithData();

            try
            {
                _context.Modelljahr = _modellJahr;
                _context.Abrechnungsjahr = _abrechJahr;
                _context.Flags = (int) LKFContextFlags.NPPlaus;
            }
            catch (LKFException ex)
            {
                Console.WriteLine(ex.Message);
                return;
            }

            _context.LoadFile(_fileName);
            uint aufenthaltCount = _context.AufenthaltCount;

            Console.WriteLine("Aufenthalte: {0}", aufenthaltCount);
            _context.PlausCheck();
            _context.Scoring();

            var npcontext = _context.NeoPaed();

            npcontext.WriteWarningFile("C:\\LKF\\out\\npwarning.txt", true);
            npcontext.WriteProtokollFile("C:\\LKF\\out\\npprotocol.txt");
            npcontext.WriteAbtFile("C:\\LKF\\out\\npabtsum.txt", true, true, true);

        }

        public void Run ()
        {
            String s = LKFController.Version;
            Console.WriteLine(string.Format("LKFXDOK Version: {0}", s));

            LKFController.DataProvider = new lkfxdok.c.LKFDataProvider(new DataProvider());

            LKFController.DataPath = "C:\\lkf\\";

            _context = new LKFContextWithData();

            try
            {
                _context.Modelljahr = _modellJahr;
                _context.Abrechnungsjahr = _abrechJahr;
                _context.Flags = 0;
            }
            catch (LKFException ex)
            {
                Console.WriteLine(ex.Message);
                return;
            }

            _context.LoadFile(_fileName);

            uint aufenthaltCount = _context.AufenthaltCount;

            Console.WriteLine("Aufenthalte: {0}", aufenthaltCount);

            _context.PlausCheck();
            _context.Scoring();
            _context.IntScoring();

            _context.WriteFile(LKFContextFileType.Error, "C:\\LKF\\out\\dotneterrors.txt");
            _context.WriteFile(LKFContextFileType.Warning, "C:\\LKF\\out\\dotnetwarnings.txt");

            for (uint i = 0; i < aufenthaltCount; i++)
            {
                var aufenthalt = _context.GetAufenthalt(i);
                Console.WriteLine(string.Format("{0}-{1}: {2}/{3}{4}: {5}", aufenthalt.KANR, aufenthalt.AUFNR, aufenthalt.PlausFlag, aufenthalt.GRUPPE, aufenthalt.KNOTEN, aufenthalt.GesamtPunkte));
            }
            /*
                var lineCount = aufenthalt.OutputLineCount;
                for (var j = 0; j < lineCount; j++)
                {
                    Console.WriteLine(aufenthalt.GetOutputLine(j));
                }

                if (aufenthalt.IsOptimizable)
                {
                    var subgruppeCount = aufenthalt.SubgruppeCount;
                    if (subgruppeCount > 1)
                    {
                        for (uint j = 0; j < subgruppeCount; j++)
                        {
                            var subgruppe = aufenthalt.GetSubgruppe(j);
                            Console.WriteLine(string.Format("     {0}{1}: {2}", subgruppe.GRUPPE, subgruppe.KNOTEN, subgruppe.Punkte));
                        }
                    }
                }
            }*/
        }

        public LKFContextWithData Context
        {
            get
            {
                return _context;
            }
        }
    }
}
